/**
 *     Caption: Zaval File Search
 *     $Revision: 1.17 $
 *     $Date: 2002/02/16 12:30:17 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.2
 *
 * The Zaval File Search solution is a local area network tool designed 
 * for fast file search on SMB shares.
 * This tool is dedicated for advanced Unix users and programmers.
 * 
 * For more info on this product read Zaval File Search User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/file-search/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0001/
 *
 * Contacts:
 *   Author : v.krapivin@zaval.org
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 * 
 * Copyright (C) 2001-2002 Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <locale.h>
#include <stdlib.h>
#include <wctype.h>
#include <wchar.h>

#define HSZ 997
#define HCONST 37

wchar_t outw[PATH_MAX+1];
char  outc[(PATH_MAX+1)*8];

static int calchash(wchar_t *s)
{
    register int x = 0;

    for(;*s;s++) {
        x = x*HCONST + towlower(*s);
    }
    x=x%(HSZ*2)+HSZ*2;
    return x % HSZ;
}

static char *chomp(register char* buf)
{
    register char* x;
    for(x=buf+strlen(buf)-1; x>buf && isspace(*x); x--);
    x[1]='\0';
    return x;
}

static char *mytok(char *begin,char delim) {
    static char *current=NULL;
    char *p,*old;

    if (begin!=NULL) current=begin;
    if (current==NULL) return NULL;
    old=current;
    p=strchr(old,delim);
    if (p!=NULL) {
        *p='\0';
        current=p+1;
    } else current=NULL;
    return old;
}

FILE* fp[HSZ];

static void openall(char *mode) {
    int i;
    char fname[PATH_MAX+1];

    for (i=0; i<HSZ; i++) {
        snprintf(fname,PATH_MAX, "%s/new/hashfiles/%i.idx",getenv("DATADIR"),i);
        if ((fp[i] = fopen(fname, mode))==NULL) perror(fname);
    }
}
static void closeall() {
    int i;
    for (i=0; i<HSZ; i++)
        fclose(fp[i]);
}
static void flushall() {
    int i;
    for (i=0; i<HSZ; i++)
        fflush(fp[i]);
}

#define cmdH 1
#define cmdF 2
//#define cmdE 3

int main(int argc,char **argv)
{
    char buf[PATH_MAX *3 + 1];

    char *type,*size,*date,*path;
    register wchar_t *ptr;

    long lines = 0;
    
    int cmd = 0,i;

    if (argc==2)
        switch (argv[1][0]) {
        case 'h': cmd=cmdH; break;
        case 'f': cmd=cmdF; break;
//        case 'e': cmd=cmdE; break;
        }

    if (cmd==0) {
        fprintf(stderr, "You should set command in command line: \n");
        fprintf(stderr, " h(ash) or f(ilenames)\n");
        return 1;
    }

    setlocale(LC_ALL, "");
    if (getenv("DATADIR")==NULL) {
        puts("You should set WDIR environment variable.");
        return;
    }

    if (cmd==cmdH) openall("wt");

    switch (cmd) {
    case cmdH:
/*  for (i=0; i<HSZ; i++) {
        fprintf(fp[i],"#\n");
        fprintf(fp[i],"# file list data for one hash value\n");
        fprintf(fp[i],"#   <type>:<date>:<size>:<path>\n");
        fprintf(fp[i],"#     where type=<F|D>, F=file, D=directory\n");
        fprintf(fp[i],"#     where date=<year>-<month>-<day>-<hour>.<minute>.<seconds>\n");
        fprintf(fp[i],"#     where size in bytes\n");
        fprintf(fp[i],"#     where path=<smb|ftp>://<hostname>/<dir>...../<filename>\n");
        fprintf(fp[i],"#\n");
    }*/
    break;
    }

    while(fgets(buf, PATH_MAX * 3, stdin)!=NULL){
        lines++;

//        if(lines%1000==0) fprintf(stderr, "%ld lines\r", lines);

        if(cmd==cmdH && lines%10000==0) flushall();

    if (buf[0]=='#') continue;
        chomp(buf);
        if(buf[0]=='\0') continue;

        type = mytok(buf, ':');
        date = mytok(NULL, ':');
        size = mytok(NULL, ':');
        path  = mytok(NULL, 0);
        if (path==NULL) {
            fprintf(stderr,"Can't detect path : %s\n",buf);
            continue;
        }

        if (mbstowcs(outw,path,PATH_MAX)==-1) {
            fprintf(stderr,"Can't convert filename to unicode : %s\n",path);
            continue;
        }
        //    Path name should be in <ftp:|smb:>//<hostname>/<path>/<filename>
        if (wcsncmp(outw,L"smb://",6)==0 && wcsncmp(outw,L"ftp://",6)==0) {
            fprintf(stderr,"Unknown protocol : %s\n",path);
            continue;
        }

        for (ptr=outw;*ptr;ptr++) *ptr=towlower(*ptr);

        switch (cmd) {
        case cmdH:
            ptr=wcsrchr(outw,L'/');
            if (ptr==NULL || ptr==outw+1) continue;
            fprintf(fp[calchash(ptr+1)], "%s:%s:%s:%s\n", type,date,size,path);
            break;
        case cmdF:
            for (ptr=outw+wcslen(outw)-1;ptr>=outw;ptr--) {
                if (*ptr==L'/') {
            wcstombs(outc,ptr+1,sizeof(outc));
            fprintf(stdout,"%s:%s\n",type,outc);
                    break;
                }
            }
            break;
/*        case cmdE:
            for (ptr=outw+wcslen(outw)-1;ptr>=outw;ptr--) {
                if (iswspace(*ptr)) break;
                if (*ptr==L'.') {
            wcstombs(outc,ptr+1,sizeof(outc));
                    break;
                }
                if (*ptr==L'/') break;
            }
            break;*/
        }
    }

    if (cmd==cmdH) closeall();
    return 0;
}
